@@ -28,7 +28,7 @@ static int WidthText(HFONT hfont, const GUI::gui_char *text) {
 	HDC hdcMeasure = ::CreateCompatibleDC(NULL);
 	HFONT hfontOriginal = static_cast<HFONT>(::SelectObject(hdcMeasure, hfont));
 	RECT rcText = {0,0, 2000, 2000};
-	::DrawText(hdcMeasure, const_cast<LPWSTR>(text), -1, &rcText, DT_CALCRECT);
+	::DrawText(hdcMeasure, text, -1, &rcText, DT_CALCRECT);
 	int width = rcText.right - rcText.left;
 	::SelectObject(hdcMeasure, hfontOriginal);
 	::DeleteDC(hdcMeasure);
@@ -1129,6 +1129,7 @@ bool FindStrip::Command(WPARAM wParam) {
 	} else {
 		pSearcher->FlagFromCmd(control) = !pSearcher->FlagFromCmd(control);
 		NextIncremental(changingEdit);
+		CheckButtons();
 	}
 	return false;
 }
@@ -1406,6 +1407,11 @@ bool ReplaceStrip::Command(WPARAM wParam) {
 	case IDREGEXP:
 	case IDUNSLASH:
 	case IDWRAP:
+	case IDM_WHOLEWORD:
+	case IDM_MATCHCASE:
+	case IDM_REGEXP:
+	case IDM_WRAPAROUND:
+	case IDM_UNSLASH:
 		pSearcher->FlagFromCmd(control) = !pSearcher->FlagFromCmd(control);
 		NextIncremental(changingEdit);
 		break;
@@ -1523,7 +1529,7 @@ void UserStrip::Size() {
 			if (ctl->controlType == UserControl::ucStatic)
 				topWithFix += 3;
 			if (ctl->controlType == UserControl::ucEdit)
-				rcSize.bottom = rcSize.top + 23;
+				rcSize.bottom = rcSize.top + lineHeight - 3;
 			if (ctl->controlType == UserControl::ucCombo)
 				rcSize.bottom = rcSize.top + 180;
 			GUI::Rectangle rcControl(left, topWithFix, left + ctl->widthAllocated, topWithFix + rcSize.Height());
@@ -1641,7 +1647,7 @@ void UserStrip::SetDescription(const char *description) {
 				puc->fixedWidth = false;
 				puc->w = ::CreateWindowEx(WS_EX_CLIENTEDGE, TEXT("Edit"), puc->text.c_str(),
 					WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS | ES_AUTOHSCROLL,
-					60 * control, line * lineHeight + space, puc->widthDesired, 27,
+					60 * control, line * lineHeight + space, puc->widthDesired, lineHeight - 3,
 					Hwnd(), HmenuID(controlID), ::GetModuleHandle(NULL), 0);
 				break;
 
